﻿using System;
using System.Linq;
using System.Threading;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using HeuristicLab.Algorithms.GeneticAlgorithm;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Reflection;

namespace HeuristicLab.Problems.MetaOptimization {
  public static class MetaOptimizationUtil {
    /// <summary>
    /// Removes those results from the run which are not declared in resultsToKeep
    /// </summary>
    public static void ClearResults(IRun run, IEnumerable<string> resultsToKeep) {
      var resultsToRemove = new List<string>();
      foreach (var result in run.Results) {
        if (!resultsToKeep.Contains(result.Key))
          resultsToRemove.Add(result.Key);
      }
      foreach (var result in resultsToRemove)
        run.Results.Remove(result);
    }

    /// <summary>
    /// Removes those parameters from the run which are not declared in parametersToKeep
    /// </summary>
    public static void ClearParameters(IRun run, IEnumerable<string> parametersToKeep) {
      var parametersToRemove = new List<string>();
      foreach (var parameter in run.Parameters) {
        if (!parametersToKeep.Contains(parameter.Key))
          parametersToRemove.Add(parameter.Key);
      }
      foreach (var parameter in parametersToRemove)
        run.Parameters.Remove(parameter);
    }

    public static double NormalizeQualities(ParameterConfigurationTree parameterConfigurationTree, double[] referenceQualities) {
      double[] qualitiesNormalized = new double[referenceQualities.Length];
      for (int i = 0; i < referenceQualities.Length; i++) {
        qualitiesNormalized[i] = parameterConfigurationTree.AverageQualities[i] / referenceQualities[i];
      }
      parameterConfigurationTree.QualitiesNormalized = new DoubleArray(qualitiesNormalized);
      parameterConfigurationTree.AverageQualityNormalized = new DoubleValue(qualitiesNormalized.Average());
      return parameterConfigurationTree.AverageQualityNormalized.Value;
    }
  }
}
